@@ -19,13 +19,10 @@ package org.apache.juli;
 
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.FilePermission;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URLClassLoader;
-import java.security.AccessControlException;
 import java.security.AccessController;
-import java.security.Permission;
 import java.security.PrivilegedAction;
 import java.util.Collections;
 import java.util.Enumeration;
@@ -298,29 +295,9 @@ public class ClassLoaderLogManager extends LogManager {
         InputStream is = null;
         // Special case for URL classloaders which are used in containers: 
         // only look in the local repositories to avoid redefining loggers 20 times
-        try {
-            if ((classLoader instanceof URLClassLoader) 
-                    && (((URLClassLoader) classLoader).findResource("logging.properties") != null)) {
-                is = classLoader.getResourceAsStream("logging.properties");
-            }
-        } catch (AccessControlException ace) {
-            // No permission to configure logging in context
-            // Log and carry on
-            ClassLoaderLogInfo info =
-                (ClassLoaderLogInfo) classLoaderLoggers.get(ClassLoader.getSystemClassLoader());
-            if (info != null) {
-                Logger log = (Logger) info.loggers.get("");
-                if (log != null) {
-                    Permission perm = ace.getPermission();
-                    if (perm instanceof FilePermission && perm.getActions().equals("read")) {
-                        log.warning("Reading " + perm.getName() + " is not permitted. See \"per context logging\" in the default catalina.policy file.");
-                    }
-                    else {
-                        log.warning("Reading logging.properties is not permitted in some context. See \"per context logging\" in the default catalina.policy file.");
-                        log.warning("Original error was: " + ace.getMessage());
-                    }
-                }
-            }
+        if ((classLoader instanceof URLClassLoader) 
+                && (((URLClassLoader) classLoader).findResource("logging.properties") != null)) {
+            is = classLoader.getResourceAsStream("logging.properties");
         }
         if ((is == null) && (classLoader == ClassLoader.getSystemClassLoader())) {
             String configFileStr = System.getProperty("java.util.logging.config.file");
@@ -475,24 +452,19 @@ public class ClassLoaderLogManager extends LogManager {
      */
     protected String replace(String str) {
         String result = str;
-        int pos_start = result.indexOf("${");
-        if (pos_start != -1) {
-            int pos_end = result.indexOf('}');
-            if (pos_end != -1) {
-                String propName = result.substring(pos_start + 2, pos_end);
+        if (result.startsWith("${")) {
+            int pos = result.indexOf('}');
+            if (pos != -1) {
+                String propName = result.substring(2, pos);
                 String replacement = System.getProperty(propName);
                 if (replacement != null) {
-                    if(pos_start >0) {
-                        result = result.substring(0,pos_start) + 
-                            replacement + replace(result.substring(pos_end + 1));
-                    } else {                       
-                        result = replacement + replace(result.substring(pos_end + 1));
-                    }
+                    result = replacement + result.substring(pos + 1);
                 }
             }
         }
         return result;
     }
+    
 
     // ---------------------------------------------------- LogNode Inner Class
 
